;==================================================================================================================
;=                                          Dark-Avenger, juin 2000                                               =
;==================================================================================================================
;=                                                                                                                =
;= Librairie d'entre/sortie sur les fichiers pour Tasm32                                                         =
;=                                                                                                                =
;= Fonctions:                                                                                                     =
;= ----------                                                                                                     =
;=                                                                                                                =
;=   - (bool)fopen              --> ouvre un fichier en lecture/criture                                          =
;=   - (bool)fclose             --> ferme un fichier                                                              =
;=   - (bool)fcreate            --> cre un fichier en lecture/criture                                           =
;=   - (bool)fread              --> lit n octets dans un fichier  partir de la position courrante                =
;=   - (bool)fwrite             --> crit n octets dans un fichier  partir de la position courrante              =
;=   - (bool)fseek_curr         --> dplace le pointeur de n octets  partir de la position courrante             =
;=   - (bool)fseek_begin        --> dplace le pointeur de n octets  partir du dbut du fichier                  =
;=   - (bool)fseek_end          --> dplace le pointeur de n octets  partir de la fin du fichier                 =
;=   - (bool)fgetsize           --> renvoie la taille du fichier                                                  =
;=   - (bool)fsetattr_any       --> rinitialise les attributs d'un fichier                                       =
;=   - (bool)fdelete            --> supprime un fichier                                                           =
;=   - (bool)frename            --> renomme un fichier                                                            =
;=   - (bool)fcopy_overwrite    --> copie un fichier dans un autre si le fichier de destination n'existe pas      =
;=   - (bool)fcopy_no_overwrite --> copie un fichier dans un autre en crasant le fichier destination s'il existe =
;=   - (bool)fsetend            --> place une marque de fin de fichier  la position courrante                    =
;=                                                                                                                =
;= Toutes les fonctions renvoient 1 dans eax si pas d'erreur, 0 sinon.                                            =
;=                                                                                                                =
;==================================================================================================================

extrn CreateFileA:proc
extrn CloseHandle:proc
extrn GetFileSize:proc
extrn SetFilePointer:proc
extrn WriteFile:proc
extrn ReadFile:proc
extrn SetFileAttributesA:proc
extrn DeleteFileA:proc
extrn MoveFileA:proc
extrn CopyFileA:proc
extrn SetEndOfFile:proc

FILE_BEGIN            equ 000000000h
FILE_CURRENT          equ 000000001h
FILE_END              equ 000000002h
OPEN_EXISTING         equ 000000003h
CREATE_ALWAYS         equ 000000002h
GENERIC_READ          equ 080000000h
GENERIC_WRITE         equ 040000000h
FILE_SHARE_READ       equ 000000001h
FILE_SHARE_WRITE      equ 000000002h
FILE_ATTRIBUTE_NORMAL equ 000000080h

;entre: eax --> pointeur sur le nom du fichier
;sortie: eax --> handle si pas erreur, 0 sinon
fopen:
  push ebx ecx edx esi edi ebp esp
  push 0
  push FILE_ATTRIBUTE_NORMAL
  push OPEN_EXISTING
  push 0
  push FILE_SHARE_READ
  push (GENERIC_READ or GENERIC_WRITE)
  push eax
  call CreateFileA
  inc eax
  cmp eax, 0
  jne fopen_Exit
fopen_Error:
  xor eax, eax
  inc eax
fopen_Exit:
  dec eax
  pop esp ebp edi esi edx ecx ebx
  ret

;entre: eax --> handle du fichier
;sortie: eax --> 1 si pas erreur, 0 sinon
fclose:
  push ebx ecx edx esi edi ebp esp
  push eax
  call CloseHandle
  cmp eax, 0
  je fclose_Exit
  xor eax, eax
  inc eax
fclose_Exit:
  pop esp ebp edi esi edx ecx ebx
  ret

;entre: eax --> pointeur sur le nom du fichier
;sortie: eax --> handle si pas erreurs, 0 sinon
fcreate:
  push ebx ecx edx esi edi ebp esp
  push 0
  push FILE_ATTRIBUTE_NORMAL
  push CREATE_ALWAYS
  push 0
  push FILE_SHARE_READ
  push (GENERIC_READ or GENERIC_WRITE)
  push eax
  call CreateFileA
  inc eax
  cmp eax, 0
  jne fcreate_Exit
fcreate_Error:
  xor eax, eax
  inc eax
fcreate_Exit:
  dec eax
  pop esp ebp edi esi edx ecx ebx
  ret

;entre: eax --> handle du fichier
;        ebx --> nombre d'octets  lire
;        ecx --> @ buffer o placer les octets
;sortie: eax --> 1 si pas erreur, 0 sinon
fread:
  push edx esi edi ebp esp
  lea edx, dword ptr[ebp]
  push 0
  push edx
  push ebx
  push ecx
  push eax
  call ReadFile
  cmp eax, 0
  je fread_Exit
  xor eax, eax
  inc eax
fread_Exit:
  pop esp ebp edi esi edx
  ret

;entre: eax --> handle du fichier
;        ebx --> nombre d'octets  crire
;        ecx --> @ buffer o sont placs les octets  crire
;sortie: eax --> 1 si pas erreur, 0 sinon
fwrite:
  push edx esi edi ebp esp
  lea edx, dword ptr[ebp]
  push 0
  push edx
  push ebx
  push ecx
  push eax
  call WriteFile
  cmp eax, 0
  je fwrite_Exit
  xor eax, eax
  inc eax
fwrite_Exit:
  pop esp ebp edi esi edx
  ret
  ret

;entre: eax --> handle du fichier
;        ebx --> nouvelle position  partir du dbut du fichier
;sortie: eax --> nouvelle position si pas erreur, 0 sinon
fseek_curr:
  push ecx edx esi edi ebp esp
  push FILE_CURRENT
  push 0
  push ebx
  push eax
  call SetFilePointer
  inc eax
  cmp eax, 0
  jne fseek_Exit
fseek_Error:
  xor eax, eax
  inc eax
fseek_Exit:
  dec eax
  pop esp ebp edi esi edx ecx
  ret

;entre: eax --> handle du fichier
;        ebx --> nouvelle position  partir de la position courrante
;sortie: eax --> nouvelle position si pas erreur, 0 sinon
fseek_begin:
  push ecx edx esi edi ebp esp
  push FILE_BEGIN
  push 0
  push ebx
  push eax
  call SetFilePointer
  inc eax
  cmp eax, 0
  jne fseek_begin_Exit
fseek_begin_Error:
  xor eax, eax
  inc eax
fseek_begin_Exit:
  dec eax
  pop esp ebp edi esi edx ecx
  ret

;entre: eax --> handle du fichier
;        ebx --> nouvelle position  partir de la fin du fichier
;sortie: eax --> nouvelle position si pas erreur, 0 sinon
fseek_end:
  push ecx edx esi edi ebp esp
  push FILE_END
  push 0
  push ebx
  push eax
  call SetFilePointer
  inc eax
  cmp eax, 0
  jne fseek_begin_Exit
fseek_end_Error:
  xor eax, eax
  inc eax
fseek_end_Exit:
  dec eax
  pop esp ebp edi esi edx ecx
  ret

;entre: eax --> handle du fichier
;sortie: eax --> taille du fichier en octets, 0 sinon
fgetsize:
  push ebx ecx edx esi edi ebp esp
  push 0
  push eax
  call GetFileSize
  inc eax
  cmp eax, 0
  jne fgetsize_Exit
fgetsize_Error:
  xor eax, eax
  inc eax
fgetsize_Exit:
  dec eax
  pop esp ebp edi esi edx ecx ebx
  ret

;entre: eax --> pointeur sur le nom du fichier
;sortie: eax --> 1 si pas erreur, 0 sinon
fsetattr_any:
  push ebx ecx edx esi edi ebp esp
  push 80h
  push eax
  call SetFileAttributesA
  cmp eax, 0
  je fsetattr_any_Exit
  xor eax, eax
  inc eax
fsetattr_any_Exit:
  pop esp ebp edi esi edx ecx ebx
  ret

;entre: eax --> pointeur sur le nom du fichier
;sortie: eax --> 1 si pas erreur, 0 sinon
fdelete:
  push ebx ecx edx esi edi ebp esp
  push eax
  call DeleteFileA
  cmp eax, 0
  jne fdelete_No_Error
fdelete_Error:
  jmp fdelete_Exit
fdelete_No_Error:
  xor eax, eax
  inc eax
fdelete_Exit:
  pop esp ebp edi esi edx ecx ebx
  ret

;entre: eax --> pointeur sur le nom du fichier
;        ebx --> pointeur sur le nouveau nom du fichier
;sortie: eax --> 1 si pas erreur, 0 sinon
frename:
  push ecx edx esi edi ebp esp
  push ebx
  push eax
  call MoveFileA
  cmp eax, 0
  jne frename_No_Error
frename_Error:
  jmp frename_Exit
frename_No_Error:
  xor eax, eax
  inc eax
frename_Exit:
  pop esp ebp edi esi edx ecx
  ret

;entre: eax --> pointeur sur le nom du fichier
;        ebx --> pointeur sur le nouveau nom du fichier
;sortie: eax --> 1 si pas erreur, 0 sinon
fcopy_overwrite:
  push ecx edx esi edi ebp esp
  push 0
  push ebx
  push eax
  call CopyFileA
  cmp eax, 0
  jne fcopy_overwrite_No_Error
fcopy_overwrite_Error:
  jmp fcopy_overwrite_Exit
fcopy_overwrite_No_Error:
  xor eax, eax
  inc eax
fcopy_overwrite_Exit:
  pop esp ebp edi esi edx ecx
  ret

;entre: eax --> pointeur sur le nom du fichier
;        ebx --> pointeur sur le nouveau nom du fichier
;sortie: eax --> 1 si pas erreur, 0 sinon
fcopy_no_overwrite:
  push ecx edx esi edi ebp esp
  push 1
  push ebx
  push eax
  call CopyFileA
  cmp eax, 0
  jne fcopy_no_overwrite_No_Error
fcopy_no_overwrite_Error:
  jmp fcopy_no_overwrite_Exit
fcopy_no_overwrite_No_Error:
  xor eax, eax
  inc eax
fcopy_no_overwrite_Exit:
  pop esp ebp edi esi edx ecx
  ret

;entre: eax --> handle du fichier
;sortie: eax --> 1 si pas erreur, 0 sinon
fsetend:
  push ebx ecx edx esi edi ebp esp
  push eax
  call SetEndOfFile
  cmp eax, 0
  jne fsetend_No_Error
fsetend_Error:
  jmp fsetend_Exit
fsetend_No_Error:
  xor eax, eax
  inc eax
fsetend_Exit:
  pop esp ebp edi esi edx ecx ebx
  ret
