;======================================================================================
;=                                                                                    =
;= (DWORD)GarbageGenerator(DWORD *pBuffer,                                            =
;=                         DWORD iBufferSize,                                         =
;=                         DWORD UsedRegs1,                                           =
;=                         DWORD UsedRegs2,                                           =
;=                         DWORD UnusedRegs1,                                         =
;=                         DWORD UnusedRegs2,                                         =
;=                         DWORD *pRNG,                                               =
;=                         DWORD *piSeed,                                             =
;=                         DWORD iPrimeNumber1,                                       =
;=                         DWORD iPrimeNumber2                                        =
;=                        )                                                           =
;=                                                                                    =
;= Paramtres:                                                                        =
;= -----------                                                                        =
;=                                                                                    =
;= DWORD *pBuffer      --> pointeur sur le buffer o placer les instructions gnres =
;= DWORD iBufferSize   --> taille du buffer en octets                                 =
;= DWORD UsedRegs1     --> registres  ne pas dtruire                                =
;= DWORD UsedRegs2     --> registres  ne pas dtruire (suite)                        =
;= DWORD UnusedRegs1   --> registres pouvant tre dtruits                            =
;= DWORD UnusedRegs2   --> registres pouvant tre dtruits (suite)                    =
;= DWORD *pRNG         --> pointeur sur un gnrateur de nombres alatoires           =
;= DWORD *piSeed        --> pointeur sur une graine                                   =
;= DWORD iPrimeNumber1 --> grand nombre premier                                       =
;= DWORD iPrimeNumber2 --> grand nombre premier (<  iPrimeNumber1)                   =
;=                                                                                    =
;= Retours:                                                                           =
;= --------                                                                           =
;=                                                                                    =
;= eax == GG_SUCCESS si pas d'erreurs                                                 =
;= eax == GG_ERROR si erreurs                                                         =
;=                                                                                    =
;= Remarques:                                                                         =
;= ----------                                                                         =
;=                                                                                    =
;= Le format du gnrateur de nombre alatoire doit tre le suivant:                  =
;=                                                                                    =
;=               (DWORD)RndNG(DWORD *piSeed,                                          =
;=                            DWORD iPrimeNumber1,                                    =
;=                            DWORD iPrimeNumber2                                     =
;=                           )                                                        =
;=                                                                                    =
;= Le format de UsedRegs1/2 et UnusedRegs1/2 est:                                     =
;=                                                                                    =
;=               xxh xxh xxh xxh                                                      =
;=                                                                                    =
;= xxh = 90h si 'blanc'. Sinon: xxh = 00h si eax                                      =
;=                              xxh = 01h si ecx                                      =
;=                              xxh = 02h si edx                                      =
;=                              xxh = 03h si ebx                                      =
;=                              xxh = 04h si esp                                      =
;=                              xxh = 05h si ebp                                      =
;=                              xxh = 06h si esi                                      =
;=                              xxh = 07h si edi                                      =
;=                                                                                    =
;= Pile:                                                                              =
;= -----                                                                              =
;=                                                                                    =
;= ESP --> CmptestState  -34
;=         EaxState      -30                                                          =
;=         PushaState    -2C                                                          =
;=         JXxx          -28                                                          =
;=         BytesLeft     -24                                                          =
;=         _edi          -20                                                          =
;=         _esi          -1C                                                          =
;=         _ebp          -18                                                          =
;=         _esp          -14                                                          =
;=         _ebx          -10                                                          =
;=         _edx          -0C                                                          =
;=         _ecx          -08                                                          =
;=         _eax          -04                                                          =
;= EBP --> ebp           +00                                                          =
;=         @ de retour   +04                                                          =
;=         *pBuffer      +08                                                          =
;=         iBufferSize   +0C                                                          =
;=         UsedRegs1     +10                                                          =
;=         UsedRegs2     +14                                                          =
;=         UnusedRegs1   +18                                                          =
;=         UnusedRegs2   +1C                                                          =
;=         *pRNG         +20                                                          =
;=         *piSeed       +24                                                          =
;=         iPrimeNumber1 +28                                                          =
;=         iPrimeNumber2 +2C                                                          =
;=                                                                                    =
;======================================================================================

include GgHeader.inc
include InstrMaker.inc

CmpTestState  = -34h
EaxState      = -30h ;1 eax destructible
PushaState    = -2Ch
JXxx          = -28h
BytesLeft     = -24h
_edi          = -20h
_esi          = -1Ch
_ebp          = -18h
_esp          = -14h
_ebx          = -10h
_edx          = -0Ch
_ecx          = -08h
_eax          = -04h
pBuffer       = +08h
iBufferSize   = +0Ch
UsedRegs1     = +10h
UsedRegs2     = +14h
UnusedRegs1   = +18h
UnusedRegs2   = +1Ch
pRNG          = +20h
piSeed        = +24h
iPrimeNumber1 = +28h
iPrimeNumber2 = +2Ch

;======================================================================================
; procdure principale
;======================================================================================
GarbageGenerator:                       ;
  push ebp                              ;
  mov ebp, esp                          ;
  pusha                                 ;
  sub esp, 14h                          ;
  mov edi, [ebp+pBuffer]                ;
  mov ecx, [ebp+iBufferSize]            ;
  mov [ebp+BytesLeft], ecx              ;
  call IsEaxDestroyable                 ;
  mov [ebp+EaxState], eax               ;
  mov dword ptr[ebp+PushaState], 0      ;
  mov dword ptr[ebp+CmpTestState], 0    ;
  mov dword ptr[ebp+JXxx], 0            ;
GarbageGenerator_Loop:                  ;
  cmp dword ptr[ebp+BytesLeft], 8       ;
  jae Anyone                            ;
  cmp dword ptr[ebp+BytesLeft], 7       ;
  jae Anyone8                           ;
  cmp dword ptr[ebp+BytesLeft], 6       ;
  jae Anyone87                          ;
  cmp dword ptr[ebp+BytesLeft], 5       ;
  jae Anyone876                         ;
  cmp dword ptr[ebp+BytesLeft], 4       ;
  jae Anyone8765                        ;
  cmp dword ptr[ebp+BytesLeft], 3       ;
  jae Anyone87654                       ;
  cmp dword ptr[ebp+BytesLeft], 2       ;
  jae Anyone876543                      ;
  cmp dword ptr[ebp+BytesLeft], 1       ;
  jae Anyone8765432                     ;
  jmp GarbageGenerator_Exit             ;
GarbageGenerator_Error:                 ;
  mov dword ptr[ebp+_eax], GG_ERROR     ;
GarbageGenerator_Exit:                  ;
  add esp, 14h                          ;
  popa                                  ;
  pop ebp                               ;
  ret 28h                               ;
                                        ;
;======================================================================================
; gnre une instruction de 1 octet
;======================================================================================
GG_IncReg:                              ;
  call dCall                            ;
  mov ebx, eax                          ;
  call IncReg                           ;
  ret                                   ;
GG_DecReg:                              ;
  call dCall                            ;
  mov ebx, eax                          ;
  call DecReg                           ;
  ret                                   ;
GG_XchgEaxReg:                          ;
  cmp dword ptr[ebp+EaxState], 1        ;
  jne GG_Int03                          ;
  call dCall                            ;
  call XchgEaxReg                       ;
  ret                                   ;
GG_Pusha:                               ;
  inc dword ptr[ebp+PushaState]         ;
  call _Pusha                           ;
  ret                                   ;
GG_Nop:                                 ;
  call _Nop                             ;
  ret                                   ;
GG_Int03:                               ;
  call _Int03                           ;
  ret                                   ;
GG_Clc:                                 ;
  call _Clc                             ;
  ret                                   ;
GG_Stc:                                 ;
  call _Stc                             ;
  ret                                   ;
                                        ;
;======================================================================================
; gnre une instruction de 2 octets
;======================================================================================
GG_AddRegReg:                           ;
  call GetSrcDest                       ;
  call AddRegReg                        ;
  ret                                   ;
GG_SubRegReg:                           ;
  call GetSrcDest                       ;
  call SubRegReg                        ;
  ret                                   ;
GG_XorRegReg:                           ;
  call GetSrcDest                       ;
  call XorRegReg                        ;
  ret                                   ;
GG_AndRegReg:                           ;
  call GetSrcDest                       ;
  call MovRegReg                        ;
  ret                                   ;
GG_OrRegReg:                            ;
  call GetSrcDest                       ;
  call OrRegReg                         ;
  ret                                   ;
GG_MovRegReg:                           ;
  call GetSrcDest                       ;
  call MovRegReg                        ;
  ret                                   ;
GG_XchgRegReg:                          ;
  call dCall                            ;
GG_XchgLoop:                            ;
  push eax                              ;
  call dCall                            ;
  pop ebx                               ;
  cmp eax, ebx                          ;
  je GG_XchgLoop                        ;
  call XchgRegReg                       ;
  ret                                   ;
GG_PushReg:                             ;
  call rCall                            ;
  call PushReg                          ;
  call dCall                            ;
  mov ebx, eax                          ;
  call PopReg                           ;
  ret                                   ;
GG_RclReg1:                             ;
  call dCall                            ;
  mov ebx, eax                          ;
  call RclReg1                          ;
  ret                                   ;
GG_RclRegCl:                            ;
  call dCall                            ;
  mov ebx, eax                          ;
  call RclRegCl                         ;
  ret                                   ;
GG_RcrReg1:                             ;
  call dCall                            ;
  mov ebx, eax                          ;
  call RcrReg1                          ;
  ret                                   ;
GG_RcrRegCl:                            ;
  call dCall                            ;
  mov ebx, eax                          ;
  call RcrRegCl                         ;
  ret                                   ;
GG_RolReg1:                             ;
  call dCall                            ;
  mov ebx, eax                          ;
  call RolReg1                          ;
  ret                                   ;
GG_RolRegCl:                            ;
  call dCall                            ;
  mov ebx, eax                          ;
  call RolRegCl                         ;
  ret                                   ;
GG_RorReg1:                             ;
  call dCall                            ;
  mov ebx, eax                          ;
  call RorReg1                          ;
  ret                                   ;
GG_RorRegCl:                            ;
  call dCall                            ;
  mov ebx, eax                          ;
  call RorRegCl                         ;
  ret                                   ;
GG_ShlReg1:                             ;
  call dCall                            ;
  mov ebx, eax                          ;
  call ShlReg1                          ;
  ret                                   ;
GG_ShlRegCl:                            ;
  call dCall                            ;
  mov ebx, eax                          ;
  call ShlRegCl                         ;
  ret                                   ;
GG_ShrReg1:                             ;
  call dCall                            ;
  mov ebx, eax                          ;
  call ShrReg1                          ;
  ret                                   ;
GG_ShrRegCl:                            ;
  call dCall                            ;
  mov ebx, eax                          ;
  call ShrRegCl                         ;
  ret                                   ;
GG_SalReg1:                             ;
  call dCall                            ;
  mov ebx, eax                          ;
  call SalReg1                          ;
  ret                                   ;
GG_SalRegCl:                            ;
  call dCall                            ;
  mov ebx, eax                          ;
  call SalRegCl                         ;
  ret                                   ;
GG_SarReg1:                             ;
  call dCall                            ;
  mov ebx, eax                          ;
  call SarReg1                          ;
  ret                                   ;
GG_SarRegCl:                            ;
  call dCall                            ;
  mov ebx, eax                          ;
  call SarRegCl                         ;
  ret                                   ;
GG_MulReg:                              ;
  cmp dword ptr[ebp+EaxState], 1        ;
  jne GG_MInt03                         ;
  call rCall                            ;
  mov ebx, eax                          ;
  call MulReg                           ;
  ret                                   ;
GG_MInt03:                              ;
  call GG_Int03                         ;
  call GG_Int03                         ;
  ret                                   ;
GG_NegReg:                              ;
  call dCall                            ;
  mov ebx, eax                          ;
  call NegReg                           ;
  ret                                   ;
GG_NotReg:                              ;
  call dCall                            ;
  mov ebx, eax                          ;
  call NotReg                           ;
  ret                                   ;
GG_Jmp:                                 ;
  cmp dword ptr[ebp+JXxx], 2            ;
  jle JXxxType2                         ;
  mov al, 0EBh                          ;
  stosb                                 ;
  jmp RndJXxx                           ;
GG_JXxx:                                ;
  cmp dword ptr[ebp+JXxx], 2            ;
  ja GG_MovRegReg                       ;
  push 70717273h                        ;
  push 74757677h                        ;
  push 78797A7Bh                        ;
  push 7C7D7E7Fh                        ;
  call RndCall                          ;
  and eax, 0Fh                          ;
  add eax, esp                          ;
  mov al, byte ptr[eax]                 ;
  stosb                                 ;
  add esp, 10h                          ;
JXxxType:                               ;
  cmp dword ptr[ebp+JXxx], 2            ;
  jae RndJXxx                           ;
  dec edi                               ;
  inc dword ptr[ebp+JXxx]               ;
  mov ax, 840Fh                         ;
  stosw                                 ;
  sub dword ptr[ebp+BytesLeft], 4       ;
  mov eax, [ebp+BytesLeft]              ;
  stosd                                 ;
JXxxType2:                              ;
  inc dword ptr[ebp+JXxx]               ;
  sub dword ptr[ebp+BytesLeft], 4       ;
  mov al, 0E9h                          ;
  stosb                                 ;
  mov eax, [ebp+BytesLeft]              ;
  dec eax                               ;
  stosd                                 ;
  ret                                   ;
RndJXxx:                                ;
  call RndCall                          ;
  stosb                                 ;
  ret                                   ;
                                        ;
;======================================================================================
; gnre une instruction de 3 octets
;======================================================================================
GG_RclRegImm8:                          ;
  call GetDestRnd                       ;
  call RclRegImm8                       ;
  ret                                   ;
GG_RcrRegImm8:                          ;
  call GetDestRnd                       ;
  call RclRegImm8                       ;
  ret                                   ;
GG_RolRegImm8:                          ;
  call GetDestRnd                       ;
  call RclRegImm8                       ;
  ret                                   ;
GG_RorRegImm8:                          ;
  call GetDestRnd                       ;
  call RclRegImm8                       ;
  ret                                   ;
GG_ShlRegImm8:                          ;
  call GetDestRnd                       ;
  call RclRegImm8                       ;
  ret                                   ;
GG_ShrRegImm8:                          ;
  call GetDestRnd                       ;
  call RclRegImm8                       ;
  ret                                   ;
GG_SalRegImm8:                          ;
  call GetDestRnd                       ;
  call RclRegImm8                       ;
  ret                                   ;
GG_SarRegImm8:                          ;
  call GetDestRnd                       ;
  call RclRegImm8                       ;
  ret                                   ;
GG_Jcxz:                                ;
  mov ax, 0E367h                        ;
  stosw                                 ;
  jmp JXxxType                          ;
                                        ;
;======================================================================================
; gnre une instruction de 4 octets
;======================================================================================
GG_CmpRegReg:                           ;
  cmp dword ptr[ebp+CmpTestState], 2    ;
  ja GG_DivReg                          ;
  inc dword ptr[ebp+CmpTestState]       ;
  call GetSrcDest                       ;
  call CmpRegReg                        ;
  call GG_JXxx                          ;
  ret                                   ;
GG_TestRegReg:                          ;
  cmp dword ptr[ebp+CmpTestState], 2    ;
  ja GG_DivReg                          ;
  inc dword ptr[ebp+CmpTestState]       ;
  call GetSrcDest                       ;
  call TestRegReg                       ;
  call GG_JXxx                          ;
  ret                                   ;
GG_DivReg:                              ;
  cmp dword ptr[ebp+EaxState], 1        ;
  jne GG_DInt03                         ;
  mov eax, 02h                          ;
  call PushReg                          ;
  call rCall                            ;
  mov ebx, eax                          ;
  call DivReg                           ;
  mov ebx, 02h                          ;
  call PopReg                           ;
  ret                                   ;
GG_DInt03:                              ;
  call GG_Int03                         ;
  call GG_Int03                         ;
  call GG_Int03                         ;
  call GG_Int03                         ;
  ret                                   ;
                                        ;
;======================================================================================
; gnre une instruction de 5 octets
;======================================================================================
GG_MovRegImm32:                         ;
  call GetDestRnd                       ;
  call MovRegImm32                      ;
  ret                                   ;
GG_AddEaxImm32:                         ;
  cmp dword ptr[ebp+EaxState], 1        ;
  jne GG_MovRegImm32                    ;
  call RndCall                          ;
  mov ecx, eax                          ;
  call AddEaxImm32                      ;
  ret                                   ;
GG_SubEaxImm32:                         ;
  cmp dword ptr[ebp+EaxState], 1        ;
  jne GG_MovRegImm32                    ;
  call RndCall                          ;
  mov ecx, eax                          ;
  call SubEaxImm32                      ;
  ret                                   ;
GG_XorEaxImm32:                         ;
  cmp dword ptr[ebp+EaxState], 1        ;
  jne GG_MovRegImm32                    ;
  call RndCall                          ;
  mov ecx, eax                          ;
  call XorEaxImm32                      ;
  ret                                   ;
GG_AndEaxImm32:                         ;
  cmp dword ptr[ebp+EaxState], 1        ;
  jne GG_MovRegImm32                    ;
  call RndCall                          ;
  mov ecx, eax                          ;
  call AndEaxImm32                      ;
  ret                                   ;
GG_OrEaxImm32:                          ;
  cmp dword ptr[ebp+EaxState], 1        ;
  jne GG_MovRegImm32                    ;
  call RndCall                          ;
  mov ecx, eax                          ;
  call OrEaxImm32                       ;
  ret                                   ;
                                        ;
;======================================================================================
; gnre une instruction de 6 octets
;======================================================================================
GG_OrRegImm32:                          ;
  call GetDestRnd                       ;
  call OrRegImm32                       ;
  ret                                   ;
GG_AndRegImm32:                         ;
  call GetDestRnd                       ;
  call AndRegImm32                      ;
  ret                                   ;
GG_XorRegImm32:                         ;
  call GetDestRnd                       ;
  call XorRegImm32                      ;
  ret                                   ;
GG_SubRegImm32:                         ;
  call GetDestRnd                       ;
  call SubRegImm32                      ;
  ret                                   ;
GG_AddRegImm32:                         ;
  call GetDestRnd                       ;
  call AddRegImm32                      ;
  ret                                   ;
                                        ;
;======================================================================================
; gnre une instruction de 7 octets
;======================================================================================
GG_TestEaxImm32:                        ;
  cmp dword ptr[ebp+CmpTestState], 2    ;
  ja GG_CmpTestReg                      ;
  inc dword ptr[ebp+CmpTestState]       ;
  call RndCall                          ;
  mov ecx, eax                          ;
  call TestEaxImm32                     ;
  call GG_JXxx                          ;
  ret                                   ;
GG_CmpEaxImm32:                         ;
  cmp dword ptr[ebp+CmpTestState], 2    ;
  ja GG_CmpTestReg                      ;
  inc dword ptr[ebp+CmpTestState]       ;
  call RndCall                          ;
  mov ecx, eax                          ;
  call CmpEaxImm32                      ;
  call GG_JXxx                          ;
  ret                                   ;
GG_CmpTestReg:                          ;
  call MovRegReg                        ;
  call CmpRegReg                        ;
  call RndCall                          ;
  and eax, 04h                          ;
  test eax, eax                         ;
  jz GG_MovRegReg                       ;
  cmp eax, 1                            ;
  je GG_RolReg1                         ;
  cmp eax, 2                            ;
  je GG_ShrRegCl                        ;
  cmp eax, 3                            ;
  je GG_SalRegCl                        ;
  cmp eax, 4                            ;
  je GG_RclReg1                         ;
  call GG_Int03                         ;
  ret                                   ;
                                        ;
;======================================================================================
; gnre une instruction de 8 octets
;======================================================================================
GG_TestRegImm32:                        ;
  cmp dword ptr[ebp+CmpTestState], 2    ;
  ja GG_TestCmp                         ;
  call GetDestRnd                       ;
  call TestRegImm32                     ;
  jmp GG_JXxx                           ;
GG_CmpRegImm32:                         ;
  cmp dword ptr[ebp+CmpTestState], 2    ;
  ja GG_TestCmp                         ;
  call GetDestRnd                       ;
  call TestRegImm32                     ;
  jmp GG_JXxx                           ;
GG_PushImm32:                           ;
  call RndCall                          ;
  mov ecx, eax                          ;
  call PushImm32                        ;
  call dCall                            ;
  mov ebx, eax                          ;
  call PopReg                           ;
  ret                                   ;
GG_TestCmp:                             ;
  call GetBoolean                       ;
  test eax, eax                         ;
  jz @1                                 ;
  call GG_MovRegReg                     ;
  jmp @Next                             ;
@1:                                     ;
  call GetBoolean                       ;
  test eax, eax                         ;
  jz @2                                 ;
  call GG_AddRegReg                     ;
  jmp @Next                             ;
@2:                                     ;
  call GetBoolean                       ;
  test eax, eax                         ;
  jz @3                                 ;
  call GG_SubRegReg                     ;
  jmp @Next                             ;
@3:                                     ;
  call GetBoolean                       ;
  test eax, eax                         ;
  jz @4                                 ;
  call GG_AndRegReg                     ;
  jmp @Next                             ;
@4:                                     ;
  call GetBoolean                       ;
  test eax, eax                         ;
  jz @5                                 ;
  call GG_XorRegReg                     ;
  jmp @Next                             ;
@5:                                     ;
  call GetBoolean                       ;
  test eax, eax                         ;
  jz @6                                 ;
  call GG_OrRegReg                      ;
  jmp @Next                             ;
@6:                                     ;
  call GetBoolean                       ;
  test eax, eax                         ;
  jz @7                                 ;
  call GG_NotReg                        ;
  jmp @Next                             ;
@7:                                     ;
  call GG_NegReg                        ;
@Next:                                  ;
  call GetBoolean                       ;
  test eax, eax                         ;
  jz @8                                 ;
  call GG_AddRegImm32                   ;
  ret                                   ;
@8:                                     ;
  call GetBoolean                       ;
  test eax, eax                         ;
  jz @9                                 ;
  call GG_SubRegImm32                   ;
  ret                                   ;
@9:                                     ;
  call GetBoolean                       ;
  test eax, eax                         ;
  jz @10                                ;
  call GG_AndRegImm32                   ;
  ret                                   ;
@10:                                    ;
  call GetBoolean                       ;
  test eax, eax                         ;
  jz @11                                ;
  call GG_XorRegImm32                   ;
  ret                                   ;
@11:                                    ;
  call GG_OrRegImm32                    ;
  ret                                   ;
                                        ;
;======================================================================================
; interface pour le choix du type d'instruction  gnrer
;======================================================================================
Anyone:                                 ;
  call GetBoolean                       ;
  test eax, eax                         ;
  jz Anyone8                            ;
  call MakeEightBytes                   ;
  jmp GarbageGenerator_Loop             ;
Anyone8:                                ;
  call GetBoolean                       ;
  test eax, eax                         ;
  jz Anyone87                           ;
  call MakeSevenBytes                   ;
  jmp GarbageGenerator_Loop             ;
Anyone87:                               ;
  call GetBoolean                       ;
  test eax, eax                         ;
  jz Anyone876                          ;
  call MakeSixBytes                     ;
  jmp GarbageGenerator_Loop             ;
Anyone876:                              ;
  call GetBoolean                       ;
  test eax, eax                         ;
  jz Anyone87654                        ;
  call MakeFiveBytes                    ;
  jmp GarbageGenerator_Loop             ;
Anyone8765:                             ;
  call GetBoolean                       ;
  test eax, eax                         ;
  jz Anyone87654                        ;
  call MakeFourBytes                    ;
  jmp GarbageGenerator_Loop             ;
Anyone87654:                            ;
  call GetBoolean                       ;
  test eax, eax                         ;
  jz Anyone876543                       ;
  call MakeThreeBytes                   ;
  jmp GarbageGenerator_Loop             ;
Anyone876543:                           ;
  call GetBoolean                       ;
  test eax, eax                         ;
  jz Anyone8765432                      ;
  call MakeTwoBytes                     ;
  jmp GarbageGenerator_Loop             ;
Anyone8765432:                          ;
  call GetBoolean                       ;
  test eax, eax                         ;
  jz AnyoneExit                         ;
  call MakeOneByte                      ;
AnyoneExit:                             ;
  jmp GarbageGenerator_Loop             ;
                                        ;
;======================================================================================
; rcupre un registre destination et un nombre alatoire
;======================================================================================
GetDestRnd:                             ;
  call RndCall                          ;
  push eax                              ;
  call dCall                            ;
  pop ecx                               ;
  mov ebx, eax                          ;
  ret                                   ;
                                        ;
;======================================================================================
; rcupre un registre source et destination
;======================================================================================
GetSrcDest:                             ;
  call dCall                            ;
  push eax                              ;
  call rCall                            ;
  pop ebx                               ;
  ret                                   ;
                                        ;
;======================================================================================
; vrifie si eax est destructible
;======================================================================================
IsEaxDestroyable:                       ;
  mov ecx, ebp                          ;
  add ecx, UnusedRegs1                  ;
  mov edx, ecx                          ;
  add edx, 4                            ;
  xor eax, eax                          ;
IsLoop:                                 ;
  mov al, byte ptr[ecx]                 ;
  cmp al, 00h                           ;
  je IsFound                            ;
  inc ecx                               ;
  cmp edx, ecx                          ;
  jne IsLoop                            ;
  xor eax, eax                          ;
  ret                                   ;
IsFound:                                ;
  xor eax, eax                          ;
  inc eax                               ;
  ret                                   ;
                                        ;
;======================================================================================
; interface pour la gnration d'une instruction de 1 octet
;======================================================================================
MakeOneByte:                            ;
  dec dword ptr[ebp+BytesLeft]          ;
  cmp dword ptr[ebp+PushaState], 1      ;
  jne MakeAnyone                        ;
  dec dword ptr[ebp+PushaState]         ;
  call _Popa                            ;
  ret                                   ;
MakeAnyone:                             ;
  call RndCall                          ;
  cmp dword ptr[ebp+BytesLeft], 07Fh    ;
  jae _MakeAnyone                       ;
  and eax, 06                           ;
  jmp _MakeNext                         ;
_MakeAnyone:                            ;
  and eax, 07                           ;
_MakeNext:                              ;
  test eax, eax                         ;
  jz GG_IncReg                          ;
  cmp eax, 1                            ;
  je GG_DecReg                          ;
  cmp eax, 2                            ;
  je GG_XchgEaxReg                      ;
  cmp eax, 3                            ;
  je GG_Int03                           ;
  cmp eax, 4                            ;
  je GG_Nop                             ;
  cmp eax, 5                            ;
  je GG_Stc                             ;
  cmp eax, 6                            ;
  je GG_Clc                             ;
  cmp eax, 7                            ;
  je GG_Pusha                           ;
  jmp GarbageGenerator_Error            ;
                                        ;
;======================================================================================
; interface pour la gnration d'une instruction de 2 octets
;======================================================================================
MakeTwoBytes:                           ;
  sub dword ptr[ebp+BytesLeft], 2       ;
  call RndCall                          ;
  and eax, 0Dh                          ;
  test eax, eax                         ;
  jz GG_AddRegReg                       ;
  cmp eax, 01h                          ;
  je GG_SubRegReg                       ;
  cmp eax, 02h                          ;
  je GG_XorRegReg                       ;
  cmp eax, 03h                          ;
  je GG_AndRegReg                       ;
  cmp eax, 04h                          ;
  je GG_OrRegReg                        ;
  cmp eax, 05h                          ;
  je GG_MovRegReg                       ;
  cmp eax, 06h                          ;
  je GG_XchgRegReg                      ;
  cmp eax, 07h                          ;
  je GG_PushReg                         ;
  cmp eax, 08h                          ;
  je GG_MulReg                          ;
  cmp eax, 09h                          ;
  je GG_NegReg                          ;
  cmp eax, 0Ah                          ;
  je GG_NotReg                          ;
  cmp eax, 0Bh                          ;
  je GG_Jmp                             ;
  cmp eax, 0Ch                          ;
  je GG_JXxx                            ;
  cmp eax, 0Dh                          ;
  je Shift                              ;
  jmp GarbageGenerator_Error            ;
Shift:                                  ;
  call RndCall                          ;
  and eax, 0Fh                          ;
  test eax, eax                         ;
  jz GG_RclReg1                         ;
  cmp eax, 01h                          ;
  je GG_RclRegCl                        ;
  cmp eax, 02h                          ;
  je GG_RcrReg1                         ;
  cmp eax, 03h                          ;
  je GG_RcrRegCl                        ;
  cmp eax, 04h                          ;
  je GG_RolReg1                         ;
  cmp eax, 05h                          ;
  je GG_RolRegCl                        ;
  cmp eax, 06h                          ;
  je GG_RorReg1                         ;
  cmp eax, 07h                          ;
  je GG_RorRegCl                        ;
  cmp eax, 08h                          ;
  je GG_ShlReg1                         ;
  cmp eax, 09h                          ;
  je GG_ShlRegCl                        ;
  cmp eax, 0Ah                          ;
  je GG_ShrReg1                         ;
  cmp eax, 0Bh                          ;
  je GG_ShrRegCl                        ;
  cmp eax, 0Ch                          ;
  je GG_SalReg1                         ;
  cmp eax, 0Dh                          ;
  je GG_SalRegCl                        ;
  cmp eax, 0Eh                          ;
  je GG_SarReg1                         ;
  cmp eax, 0Fh                          ;
  je GG_SarRegCl                        ;
  jmp GarbageGenerator_Error            ;
                                        ;
;======================================================================================
; interface pour la gnration d'une instruction de 3 octets
;======================================================================================
MakeThreeBytes:                         ;
  sub dword ptr[ebp+BytesLeft], 3       ;
  call RndCall                          ;
  and eax, 08h                          ;
  test eax, eax                         ;
  jz GG_Jcxz                            ;
  cmp eax, 1                            ;
  je GG_RclRegImm8                      ;
  cmp eax, 2                            ;
  je GG_RcrRegImm8                      ;
  cmp eax, 3                            ;
  je GG_RolRegImm8                      ;
  cmp eax, 4                            ;
  je GG_RorRegImm8                      ;
  cmp eax, 5                            ;
  je GG_ShlRegImm8                      ;
  cmp eax, 6                            ;
  je GG_ShrRegImm8                      ;
  cmp eax, 7                            ;
  je GG_SalRegImm8                      ;
  cmp eax, 8                            ;
  je GG_SarRegImm8                      ;
  jmp GarbageGenerator_Error            ;
                                        ;
;======================================================================================
; interface pour la gnration d'une instruction de 4 octets
;======================================================================================
MakeFourBytes:                          ;
  call RndCall                          ;
  and eax, 02h                          ;
  test eax, eax                         ;
  jz MakeTwoBytes                       ;
  cmp eax, 1                            ;
  je Shift                              ;
  sub dword ptr[ebp+BytesLeft], 4       ;
  call RndCall                          ;
  and eax, 02h                          ;
  test eax, eax                         ;
  jz GG_CmpRegReg                       ;
  cmp eax, 1                            ;
  je GG_TestRegReg                      ;
  cmp eax, 2                            ;
  je GG_DivReg                          ;
  jmp GarbageGenerator_Error            ;
                                        ;
;======================================================================================
; interface pour la gnration d'une instruction de 5 octets
;======================================================================================
MakeFiveBytes:                          ;
  call RndCall                          ;
  and eax, 02h                          ;
  test eax, eax                         ;
  jz MakeTwoBytes                       ;
  cmp eax, 1                            ;
  je MakeFourBytes                      ;
  sub dword ptr[ebp+BytesLeft], 5       ;
  call RndCall                          ;
  and eax, 05h                          ;
  test eax, eax                         ;
  jz GG_MovRegImm32                     ;
  cmp eax, 1                            ;
  je GG_AddEaxImm32                     ;
  cmp eax, 2                            ;
  je GG_SubEaxImm32                     ;
  cmp eax, 3                            ;
  je GG_AndEaxImm32                     ;
  cmp eax, 4                            ;
  je GG_OrEaxImm32                      ;
  cmp eax, 5                            ;
  je GG_XorEaxImm32                     ;
  jmp GarbageGenerator_Error            ;
                                        ;
;======================================================================================
; interface pour la gnration d'une instruction de 6 octets
;======================================================================================
MakeSixBytes:                           ;
  call RndCall                          ;
  and eax, 04h                          ;
  test eax, eax                         ;
  jz MakeTwoBytes                       ;
  cmp eax, 1                            ;
  je MakeThreeBytes                     ;
  cmp eax, 2                            ;
  je MakeFourBytes                      ;
  cmp eax, 3                            ;
  je MakeFiveBytes                      ;
  sub dword ptr[ebp+BytesLeft], 6       ;
  call RndCall                          ;
  and eax, 04h                          ;
  test eax, eax                         ;
  jz GG_AddRegImm32                     ;
  cmp eax, 1                            ;
  je GG_SubRegImm32                     ;
  cmp eax, 2                            ;
  je GG_XorRegImm32                     ;
  cmp eax, 3                            ;
  je GG_AndRegImm32                     ;
  cmp eax, 4                            ;
  je GG_OrRegImm32                      ;
  jmp GarbageGenerator_Error            ;
                                        ;
;======================================================================================
; interface pour la gnration d'une instruction de 7 octets
;======================================================================================
MakeSevenBytes:                         ;
  call RndCall                          ;
  and eax, 05h                          ;
  test eax, eax                         ;
  jz MakeTwoBytes                       ;
  cmp eax, 1                            ;
  je MakeThreeBytes                     ;
  cmp eax, 2                            ;
  je MakeFourBytes                      ;
  cmp eax, 3                            ;
  je MakeFiveBytes                      ;
  cmp eax, 4                            ;
  je MakeSixBytes                       ;
  sub dword ptr[ebp+BytesLeft], 7       ;
  call RndCall                          ;
  and eax, 01h                          ;
  test eax, eax                         ;
  jz GG_TestEaxImm32                    ;
  cmp eax, 1                            ;
  je GG_CmpEaxImm32                     ;
  jmp GarbageGenerator_Error            ;
                                        ;
;======================================================================================
; interface pour la gnration d'une instruction de 8 octets
;======================================================================================
MakeEightBytes:                         ;
  call RndCall                          ;
  and eax, 07h                          ;
  test eax, eax                         ;
  jz MakeOneByte                        ;
  cmp eax, 1
  jz MakeTwoBytes                       ;
  cmp eax, 2                            ;
  je MakeThreeBytes                     ;
  cmp eax, 3                            ;
  je MakeFourBytes                      ;
  cmp eax, 4                            ;
  je MakeFiveBytes                      ;
  cmp eax, 5                            ;
  je MakeSixBytes                       ;
  cmp eax, 6                            ;
  je MakeSevenBytes                     ;
  sub dword ptr[ebp+BytesLeft], 8       ;
  call RndCall                          ;
  and eax, 02h                          ;
  test eax, eax                         ;
  jz GG_TestRegImm32                    ;
  cmp eax, 1                            ;
  je GG_CmpRegImm32                     ;
  cmp eax, 2                            ;
  je GG_PushImm32                       ;
  jmp GarbageGenerator_Error            ;
                                        ;
;======================================================================================
; renvoie alatoirement un boolen
;======================================================================================
GetBoolean:                             ;
  call RndCall                          ;
  and eax, 1                            ;
  ret                                   ;
                                        ;
;======================================================================================
; interface pour l'appel au gnrateur de nombres alatoires
;======================================================================================
RndCall:                                ;
  push dword ptr[ebp+iPrimeNumber2]     ;
  push dword ptr[ebp+iPrimeNumber1]     ;
  push dword ptr[ebp+piSeed]            ;
  call dword ptr[ebp+pRNG]              ;
  ret                                   ;
                                        ;
;======================================================================================
; interface pour l'appel  dGetReg
;======================================================================================
dCall:                                  ;
  call RndCall                          ;
  push eax                              ;
  mov edx, ebp                          ;
  add edx, UnusedRegs1                  ;
  add edx, 4                            ;
  push edx                              ;
  sub edx, 4                            ;
  push edx                              ;
  call dGetReg                          ;
  ret                                   ;
                                        ;
;======================================================================================
; interface pour l'appel  rGetReg
;======================================================================================
rCall:                                  ;
  call RndCall                          ;
  push eax                              ;
  mov edx, ebp                          ;
  add edx, UsedRegs1                    ;
  add edx, 4                            ;
  push edx                              ;
  sub edx, 4                            ;
  push edx                              ;
  call rGetReg                          ;
  ret                                   ;
                                        ;
;======================================================================================
;
; slectionne un registre destructible alatoirement
;
; (DWORD)dGetReg(DWORD *pUnusedRegs1,
;                DWORD *pUnusedRegs2,
;                DWORD RndNumber
;               )
;
;======================================================================================
dGetReg:                                ;
  push ebp                              ;
  mov ebp, esp                          ;
  pusha                                 ;
  mov ebx, [ebp+10h]                    ;
  and ebx, 07h                          ;
  mov ecx, [ebp+08h]                    ;
  add ecx, ebx                          ;
  mov edx, [ebp+0Ch]                    ;
  add edx, 4                            ;
  xor eax, eax                          ;
gLoop:                                  ;
  mov al, byte ptr[ecx]                 ;
  cmp al, 90h                           ;
  jne gFound                            ;
  inc ecx                               ;
  cmp edx, ecx                          ;
  jne gLoop                             ;
  mov ecx, [ebp+08h]                    ;
  jmp gLoop                             ;
gFound:                                 ;
  mov dword ptr[ebp+_eax], eax          ;
  popa                                  ;
  pop ebp                               ;
  ret 0Ch                               ;
                                        ;
;======================================================================================
;
; slectionne un registre rel alatoirement
;
; (DWORD)rGetReg(DWORD *pUsedRegs1,
;                DWORD *pUsedRegs2,
;                DWORD RndNumber
;               )
;
;======================================================================================
rGetReg:                                ;
  push ebp                              ;
  mov ebp, esp                          ;
  pusha                                 ;
  mov ebx, [ebp+10h]                    ;
  and ebx, 07h                          ;
  mov ecx, [ebp+08h]                    ;
  add ecx, ebx                          ;
  mov edx, [ebp+0Ch]                    ;
  add edx, 4                            ;
  xor eax, eax                          ;
rLoop:                                  ;
  mov al, byte ptr[ecx]                 ;
  cmp al, 90h                           ;
  jne rFound                            ;
  inc ecx                               ;
  cmp edx, ecx                          ;
  jne rLoop                             ;
  mov ecx, [ebp+08h]                    ;
  jmp rLoop                             ;
rFound:                                 ;
  mov dword ptr[ebp+_eax], eax          ;
  popa                                  ;
  pop ebp                               ;
  ret 0Ch                               ;
                                        ;
